# SSCG - Simple Signed Certificate Generator

## Why?
x509 Certificates are complicated to get right. SSCG makes it easy to generate usable, _signed_ certificates quickly without needing to understand complex `openssl`, `certtool` or `certutil` commands.

## Quick Usage - Certificates Signed by a Private CA
```
/usr/bin/sscg
````
This will produce a `ca.crt` and a pair of `service.pem` and `service-key.pem` files in the current directory, the latter of which is signed by the private key of the `ca.crt`. The CA's private key will be destroyed by default after these files are created, so nothing further can be signed by it. (See [this blog post](https://sgallagh.wordpress.com/2016/05/02/self-signed-ssltls-certificates-why-they-are-terrible-and-a-better-alternative/)  for details on why this is desirable.)

Certificates generated by SSCG will be valid by default for securing the current machine's hostname and no others. To add further names (such as for a multi-homed system), look at the `-subject-alt-name` argument below.


## Full Usage
Usage of sscg:
```
/usr/bin/sscg [options]
  -ca-file string
    	Path where the public CA certificate will be stored. (default "ca.crt")
  -ca-key-file string
    	Path where the CA's private key will be stored. If unspecified, the key will be destroyed rather than written to the disk.
  -cert-file string
    	Path where the public service certificate will be stored. (default "service.pem")
  -cert-key-file string
    	Path where the service's private key will be stored. (default "service-key.pem")
  -country string
    	Certificate DN: Country (C) (default "US")
  -debug
    	Enable logging of debug messages. Implies verbose. Warning! This will print private key information to the screen!
  -hash-alg value
    	Hashing algorithm to use for signing. {sha256,sha384,sha512} (default sha256)
  -hostname string
    	The valid hostname of the certificate. Must be an FQDN. (default system hostname)
  -key-strength value
    	Strength of the certificate private keys in bits. {512,1024,2048,4096} (default 2048)
  -lifetime uint
    	Certificate lifetime (days). (default 3650)
  -organization string
    	Certificate DN: Organization (O) (default "Unspecified")
  -package string
    	The name of the package needing a certificate (default "Unknown")
  -quiet
    	Display no output unless there is an error.
  -signing-cert string
    	The location of an existing signing certificate. Setting this option will skip creation of a private CA.
  -signing-key string
    	The location of an existing signing key. Setting this option will skip creation of a private CA.
  -subject-alt-name value
    	An additional valid hostname for the certificate. May be specified multiple times. (default [])
  -verbose
    	Display progress messages.
  -version
    	Display the version number and exit
```
